package com.caimei365.manager.service.caimei.impl;

import com.caimei365.manager.dao.FinanceDao;
import com.caimei365.manager.entity.PaginationVo;
import com.caimei365.manager.entity.ResponseJson;
import com.caimei365.manager.entity.caimei.*;
import com.caimei365.manager.service.caimei.FinanceService;
import com.github.pagehelper.PageHelper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * Description
 *
 * @author : Charles
 * @date : 2022/3/15
 */
@Slf4j
@Service
public class FinanceServiceImpl implements FinanceService {
    @Resource
    private FinanceDao financeDao;

    /**
     * 获取收款类型
     */
    @Override
    public ResponseJson<List<ReceiptType>> getReceiptType() {
        List<ReceiptType> list = financeDao.getReceiptType();
        return ResponseJson.success(list);
    }

    /**
     * 获取收款列表
     */
    @Override
    public ResponseJson<PaginationVo<ReceiptList>> getReceiptList(ReceiptParam receiptParam, int pageNum, int pageSize) {
        if (StringUtils.hasLength(receiptParam.getStartTime())) {
            String startTime = receiptParam.getStartTime().substring(0, 10) + " 00:00:00";
            receiptParam.setStartTime(startTime);
        }
        if (StringUtils.hasLength(receiptParam.getEndTime())) {
            String endTime = receiptParam.getEndTime().substring(0, 10) + " 23:59:59";
            receiptParam.setEndTime(endTime);
        }
        PageHelper.startPage(pageNum, pageSize);
        List<ReceiptList> receiptList = financeDao.getReceiptList(receiptParam);
        PaginationVo<ReceiptList> pageData = new PaginationVo<>(receiptList);
        return ResponseJson.success(pageData);
    }

    /**
     * 收款详情
     */
    @Override
    public ResponseJson<ReceiptDetail> getReceiptDetail(Integer id) {
        ReceiptDetail receipt = financeDao.getReceiptDetail(id);
        List<ReceiptRelation> relations = financeDao.getReceiptRelations(id);
        if (!CollectionUtils.isEmpty(relations) && relations.size() > 1) {
            receipt.setReceiptOrderFlag(true);
        }
        // 一个收款可能会关联了一个或多个订单
        List<ReceiptOrder> orderInfos = new ArrayList<>();
        for (ReceiptRelation relation : relations) {
            ReceiptOrder roi = null;
            if (2 == relation.getRelationType()) {
                // relationType==2时，orderId为主订单Id
                roi = financeDao.getOrderInfoByOrderId(relation.getOrderId());
            }
            if (1 == relation.getRelationType()) {
                // relationType==1时，orderId为子订单Id
                roi = financeDao.getReceiptOrderByShopOrderId(relation.getOrderId());
                if (null != roi) {
                    if (5 == receipt.getReceiptType()) {
                        // 供应商退款,已退金额
                        double refundAmount = financeDao.getRefundAmountByShopOrder(roi.getShopOrderId());
                        roi.setRefundsAmount(refundAmount);
                    }
                }
            }
            if (null != roi) {
                roi.setRelationType(relation.getRelationType());
                // 根据订单Id获取收款记录
                List<ReceiptRecord> recordList = financeDao.getReceiptRecordByOrderId(roi.getOrderId());
                // 暂时不启用抹平明细
                if (!CollectionUtils.isEmpty(recordList)) {
                    for (ReceiptRecord record : recordList) {
                        if (3 == record.getConfirmType()) {
                            double returnBalanceAmount = financeDao.getReturnBalanceAmount(record.getReceiptId());
                            record.setReturnBalanceAmount(returnBalanceAmount);
                        }
                        // 统计当前已收
                        double associateAmount = financeDao.getAssociateAmountByOrderId(roi.getOrderId());
                        record.setOrderReceivedAmount(associateAmount);
                        if (16 == receipt.getPayType()) {
                            Double userMoney = financeDao.sumUserMoney(roi.getOrderId());
                            Double onlineMoney = financeDao.sumOnlineMoney(roi.getOrderId());
                            if (null == onlineMoney) {
                                record.setOnlineMoney(0d);
                            } else {
                                record.setOnlineMoney(onlineMoney);
                            }
                            if (null == userMoney) {
                                record.setUserMoney(0d);
                            } else {
                                record.setUserMoney(userMoney);
                            }
                        }
                    }
                }
                //一个订单又包含了多个收款记录
                roi.setReceiptRecords(recordList);
                orderInfos.add(roi);
            }
        }
        receipt.setOrderInfos(orderInfos);

        if (6 == receipt.getReceiptType()) {
            // 3超级会员款
            ReceiptVipInfo receiptVipInfo = financeDao.getReceiptVipInfo(receipt.getId());
            receipt.setVipInfo(receiptVipInfo);
        }
        if (8 == receipt.getReceiptType()) {
            //优惠券购买费
            ReceiptCouponInfo couponInfo = financeDao.getReceiptCouponInfo(receipt.getId());
            receipt.setCouponInfo(couponInfo);
        }

        return ResponseJson.success(receipt);
    }

    /**
     * 撤销收款
     */
    @Transactional(rollbackFor=Exception.class)
    @Override
    public ResponseJson<Void> cancelReceipt(ReceiptCancelParam receiptCancelParam) {
        Integer paramId = receiptCancelParam.getId();
        Integer paramOrderId = receiptCancelParam.getOrderId();
        String paramRemark = receiptCancelParam.getRemark();
        Integer sysUserId = receiptCancelParam.getSysUserId();
        ReceiptDetail receipt = financeDao.getReceiptDetail(paramId);
        //receiptType 收款款项类型：1订单款，2非订单款，3返佣款
        //1订单款
        //撤销前  如果收款记录已经审核 那么要去还原这条收款记录  如果收款记录没有通过审核 那么直接修改收款记录的状态就可以
        if (1 == receipt.getReceiptType()) {
            // 先撤销一下收款记录就好了
            if (3 == receipt.getReceiptStatus()) {  //  还原订单收款状态
                // 找到这个记录关联了那些订单  看看订单除了这一次收款还有没有其他的收款记录  如果有 那么订单会被还原成部分收款
                // 如果没有 那么订单就是待收款  注意这里的订单都是主订单
                List<Integer> orderIds = financeDao.getOrderIdsByReceiptId(receipt.getId());
                for (Integer orderId : orderIds) {
                    CmOrder order = financeDao.getOrderByOrderId(orderId);
                    // 统计订单款项的收款记录
                    int recordCount = financeDao.getOrderReceiptCount(orderId, receipt.getId());
                    if (recordCount == 0 && order.getBalancePayFee() <= 0d) {
                        //说明除了现在正在撤销的这一条收款记录  这个订单没有其他的收款记录  就是说 撤销以后这个订单是代付款状态
                        order.setReceiptStatus(1);
                        if (order.getStatus().toString().length() == 2) {
                            String status = "1" + order.getStatus().toString().charAt(1);
                            order.setStatus(Integer.valueOf(status));
                        }
                        if (4 == order.getStatus() || 5 == order.getStatus()) {
                            order.setStatus(13);
                        }
                    } else {
                        //反之这个订单就是部分付款状态
                        order.setReceiptStatus(2);
                        if (order.getStatus().toString().length() == 2) {
                            String status = "2" + order.getStatus().toString().charAt(1);
                            order.setStatus(Integer.valueOf(status));
                        }
                        if (4 == order.getStatus() || 5 == order.getStatus()) {
                            order.setStatus(23);
                        }
                    }
                    financeDao.updateOrderStatus(order);
                    // 判断是否是充值定金商品
                    int depositProduct = financeDao.countDepositOrderProduct(order.getOrderId());
                    if (depositProduct > 0) {
                        //保存余额到余额收支记录
                        BalanceRecord balanceRecord = new BalanceRecord();
                        balanceRecord.setType(2);
                        balanceRecord.setBalanceType(8);
                        balanceRecord.setAddDate(new Date());
                        balanceRecord.setAmount(order.getPayTotalFee());
                        balanceRecord.setOrderId(order.getOrderId());
                        balanceRecord.setUserId(order.getUserId());
                        balanceRecord.setReceiptId(receipt.getId());
                        balanceRecord.setRemark("定金商品撤销收款");
                        balanceRecord.setDelFlag(0);
                        // 保存 余额收支记录
                        financeDao.insertBalanceRecord(balanceRecord);
                        financeDao.updateReduceUserMoney(order.getPayTotalFee(), order.getUserId());
                    }
                }
            }
        }
        //3返佣款
        if (3 == receipt.getReceiptType()) {
            // 如果返佣收款记录还在审核中 那么直接修改一下订单状态就好了
            if (2 == receipt.getReceiptStatus()) {
                // 如果返佣收款记录还在已经确认了 那么就去看看这笔返佣收款记录关联了那些子订单  然后撤销它们的主订单的收款状态以及付款状态
                List<Integer> shopOrderIds = financeDao.getShopOrderIdsByReceiptId(receipt.getId());
                for (Integer shopOrderId : shopOrderIds) {
                    //records  查询的是 shopOrderID  这个子订单它的兄弟子订单 就是他的主订单下面除了他这个子订单 其他的子订单的返佣收款记录
                    // 统计子订单款项的收款记录
                    int recordCount = financeDao.getShopOrderReceiptCount(shopOrderId, receipt.getId());
                    CmShopOrder cmShopOrder = financeDao.getShopOrderById(shopOrderId);
                    CmOrder order = financeDao.getOrderByOrderId(cmShopOrder.getOrderId());
                    if (recordCount > 0 && order.getBalancePayFee() <= 0d) {
                        //说明除了现在正在撤销的这一条收款记录  这个订单没有其他的收款记录  就是说 撤销以后这个订单是代付款状态
                        order.setReceiptStatus(1);
                        order.setPayStatus(1);
                        cmShopOrder.setPayStatus(1);
                        if (order.getStatus().toString().length() == 2) {
                            String status = "1" + order.getStatus().toString().charAt(1);
                            order.setStatus(Integer.valueOf(status));
                        }
                        if (4 == order.getStatus() || 5 == order.getStatus()) {
                            order.setStatus(13);
                        }

                    } else {
                        //反之这个订单就是部分付款状态
                        order.setReceiptStatus(2);
                        order.setPayStatus(2);
                        cmShopOrder.setPayStatus(2);
                        if (order.getStatus().toString().length() == 2) {
                            String status = "2" + order.getStatus().toString().charAt(1);
                            order.setStatus(Integer.valueOf(status));
                        }
                        if (4 == order.getStatus() || 5 == order.getStatus()) {
                            order.setStatus(23);
                        }
                    }
                    financeDao.updateOrderStatus(order);
                    financeDao.updateShopOrderStatus(cmShopOrder);
                }
                if (10 == receipt.getPayType()) {
                    //是口头返佣  需要扣除供应商返佣的余额  删除返佣欠款记录
                    RebateRecord rebateRecord = financeDao.getRebateRecordByReceiptId(receipt.getId());
                    CmShop shop = financeDao.getShopByShopId(rebateRecord.getShopId());
                    shop.setAbleRebateAmount(shop.getAbleRebateAmount() - rebateRecord.getRebateAmount());
                    shop.setRebateAmount(shop.getRebateAmount() - rebateRecord.getRebateAmount());
                    financeDao.updateShopAbleRebateAmount(shop);
                    financeDao.deleteRebateRecord(receipt.getId());
                    receipt.setDelFlag(1);
                }
            }
        }
        String format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
        receipt.setReceiptStatus(5);
        receipt.setCancelReason(paramRemark);
        receipt.setCancelDate(format);
        receipt.setCancelUserPermissionId(sysUserId);
        financeDao.cancelReceipt(receipt);
        //撤销收款后如果存在多收款退款余额，则此处需要重新减少用户余额，以及撤销余额收支记录
        CmOrder order = financeDao.getOrderByOrderId(paramOrderId);
        if (null != receipt.getConfirmType() && 3 == receipt.getConfirmType()) {
            //撤销余额收支记录
            BalanceRecord balanceRecord = new BalanceRecord();
            balanceRecord.setOrderId(order.getOrderId());
            balanceRecord.setReceiptId(receipt.getId());
            balanceRecord.setUserId(order.getUserId());
            balanceRecord.setRemark("撤销收款，余额记录无效-后台");
            balanceRecord.setDelFlag(1);
            // 保存 余额收支记录
            financeDao.updateBalanceRecord(balanceRecord);
            // 收款金额
            Double receiptAmount = receipt.getReceiptAmount();
            // 订单真实需支付金额
            Double payableAmount = order.getPayableAmount();
            //多收退款余额金额
            Double balanceAmount = 0d;
            if (receiptAmount > payableAmount) {
                balanceAmount = receiptAmount - payableAmount;
            }
            //回收用户退款余额
            financeDao.updateReduceUserMoney(balanceAmount, order.getUserId());
        }

        return ResponseJson.success();
    }
}
